// SPDX-License-Identifier: GPL-2.0-or-later
// SPDX-FileCopyrightText: 2021 Tomaz Canabrava <tcanabrava@kde.org>

#include <QDebug>

#include "firewallclient.h"

int main()
{
    qDebug() << "SS test called";
}
